import processing.core.*; 
import processing.data.*; 
import processing.event.*; 
import processing.opengl.*; 

import processing.video.*; 

import java.util.HashMap; 
import java.util.ArrayList; 
import java.io.File; 
import java.io.BufferedReader; 
import java.io.PrintWriter; 
import java.io.InputStream; 
import java.io.OutputStream; 
import java.io.IOException; 

public class nonsquare_webcam_and_may_9_15 extends PApplet {




Capture cam;

MasterConfig current_run;

DataRectangle input_rectangle;

boolean black_and_white, inverted, fluid_reference;

int base_frame_count;

public void setup () {
  current_run = new MasterConfig();
  input_rectangle = may_8_graph(current_run);
  //may_9_graph(current_run);
  cam = new Capture(this, 640, 480, 30);
  cam.start();
  black_and_white = false;
  inverted = false;
  fluid_reference = false;
}

public void draw () {
  background (127);
    if(cam.available()) {
    cam.read();
  }
  input_rectangle.img.copy(cam, 0, 0, 640, 480, 0, 0, 640, 480);
  if (inverted) input_rectangle.img.filter(INVERT);
  if (black_and_white) input_rectangle.img.filter(GRAY);
  current_run.draw_images();
  current_run.apply_transforms();
  current_run.shift_data();
  if (fluid_reference) {
     textSize(42);
     float fade = min((frameCount-base_frame_count)/100.0f,1.0f);
     float back = 1 - fade;
     fill(back*127+fade*255, back*127, back*127+fade*255);
     text("github.com/anhinga/fluid => atparty-2018 => surreal webcam", 10, 498); 
  }
}

public void mousePressed() {
  current_run.try_controls(mouseX, mouseY);
}

public void mouseDragged() {
  current_run.try_controls(mouseX, mouseY);
}

public void keyPressed() {
  if (key == 'b' || key == 'B') black_and_white = !black_and_white;
  if (key == 'i' || key == 'I') inverted = !inverted;
  if (key == 'f' || key == 'F') {
    fluid_reference = !fluid_reference;
    base_frame_count = frameCount;
  }
}
class ClickControl {
  
  int left_x;
  int left_y;
  int control_width;
  int control_height;
 
  ClickControl(int _left_x, int _left_y, int _control_width, int _control_height) {
    left_x = _left_x;
    left_y = _left_y;
    control_width = _control_width;
    control_height = _control_height; 
  }
  
  public void click(int click_x, int click_y) {
  }  
  
}
class CustomClickControl extends ClickControl{
  int frame_count_base;
  int center_x;
  int center_y;
  
  CustomClickControl(int _left_x, int _left_y, int _control_width, int _control_height) {
    super(_left_x, _left_y, _control_width, _control_height); 
    center_x = _control_width/2;
    center_y = _control_height/2;
    frame_count_base = 0; 
  }
  
  public void click(int click_x, int click_y) {
    frame_count_base = frameCount;
    center_x = click_x;
    center_y = click_y;
  } 
}
// This class abstracts the wave transform used in my main demo, Fall 2014

class CustomWaveTransform extends Transform {
  
  DataRectangle input;
  DataRectangleDynamic output;
  CustomClickControl control;
  float distance_coef;
 
  CustomWaveTransform(DataRectangle _input, DataRectangleDynamic _output, CustomClickControl _control) {
    input = _input;
    output = _output;
    control = _control;
    distance_coef = 1.0f;
  } 
  
  public void set_distance_coef (float _distance_coef) {
    distance_coef = _distance_coef;
  } 
  
  public void apply() {
    int effective_frame_count = frameCount - control.frame_count_base;
    float center_x = control.center_x;
    float center_y = control.center_y; 
    float shift = effective_frame_count/(1+0.01f*effective_frame_count);
    int output_width = output.rect_width;
    int output_height = output.rect_height;
    int input_width = input.rect_width;
    int input_height = input.rect_height;
    for (int i = 0; i < output_width; i++)
      for (int j = 0; j < output_height; j++) {
          float d = dist(i,j,center_x,center_y);
          float mod = 0.25f*sin(distance_coef*(d/effective_frame_count+shift))+0.75f;
          int i1 = PApplet.parseInt( (center_x + (i-center_x)*mod)*input_width/output_width );
          int j1 = PApplet.parseInt( (center_y + (j-center_y)*mod)*input_height/output_height );
          int c = input.get_color_point(i1,j1);
          output.set_color_point(i,j,c);
      }    
  }
}
// file should be renamed

class DataRectangle {

  int rect_width;
  int rect_height;
  
  PImage img;

  public int get_color_point (int i, int j) {
    return img.get(i,j);
  }

  public void describe(int _width, int _height) {
    rect_width = _width;
    rect_height = _height;
    img = createImage(rect_width, rect_height, RGB);
    img.loadPixels();
    for (int i = 0; i < img.pixels.length; i++) {
      img.pixels[i] = color(127, 127, 127); 
    }
    img.updatePixels();
  }

  public void describe(int _width, int _height, String file_name) {
    rect_width = _width;
    rect_height = _height;
    img = loadImage(file_name);
    img.resize(rect_width, rect_height);
  }
  
}
//file should be renamed

class DataRectangleDynamic extends DataRectangle {
  
  PImage img_write;
  
  public void describe(int _width, int _height) {
    super.describe(_width, _height);
    img_write = img.get();
  }

  public void set_color_point (int i, int j, int c) {
    img_write.set(i,j,c);
  }
  
  public void shift_data () {
    PImage img_tmp = img;
    img = img_write;
    img_write = img_tmp; 
  }
}
// class holding the whole dataflow configuration

class MasterConfig {

  Transform[] transforms;
  DataRectangleDynamic[] rects;
  OutputDataRectangle[] images;
  ClickControl[] controls;
  
  MasterConfig () {
    transforms = new Transform[0];
    rects = new DataRectangleDynamic[0]; 
    images = new OutputDataRectangle[0];
    controls = new ClickControl[0];
  }
  
  public void register_transform(Transform _transform) {
    transforms = (Transform[]) append(transforms, _transform);
  }
  
  public void register_data(DataRectangleDynamic _data) {
    rects = (DataRectangleDynamic[]) append(rects, _data);
  }
  
  public void register_output(int _left_x, int _left_y, DataRectangle _data) {
    images = (OutputDataRectangle[]) append(images, new OutputDataRectangle(_left_x, _left_y, _data)); 
  }
  
  public void register_control(ClickControl _control) {
    controls = (ClickControl[]) append(controls, _control);
  }
  
  public void apply_transforms() {
    for (int i = 0; i < transforms.length; i++)
      transforms[i].apply(); 
  }
  
  public void shift_data() {
    for (int i = 0; i < rects.length; i++) 
      rects[i].shift_data();
  }
  
  public void draw_images() {
    for (int i = 0; i < images.length; i++) {
      OutputDataRectangle cur_img = images[i];
      image(cur_img.data.img, cur_img.left_x, cur_img.left_y);
    }
  }
  
  public void try_controls(int _mouse_x, int _mouse_y) {
    for (int i = 0; i < controls.length; i++) {
      ClickControl current = controls[i];
      if (_mouse_x >= current.left_x && 
          _mouse_y >= current.left_y &&
          _mouse_x < current.left_x + current.control_width &&
          _mouse_y < current.left_y + current.control_height) {
            
            current.click(_mouse_x - current.left_x, _mouse_y - current.left_y);
      }      
    }
     
  }  
}


public DataRectangle may_8_graph(MasterConfig this_run) {

  int hor_size = 640;  
  int vert_size = 480;
  int margin_size = 5;
  int second_hor_size = hor_size+2*margin_size;
  int second_vert_size = vert_size+2*margin_size;
  int third_hor_size = 2*hor_size+3*margin_size;
  int third_vert_size = 2*vert_size+3*margin_size;
  int slider_width = 50;

  size (third_hor_size + slider_width + margin_size, third_vert_size);
  
  DataRectangle input_1 = new DataRectangle();  
  input_1.describe(hor_size, vert_size);
  //input_1.img.filter(INVERT);
  //input_1.img.filter(GRAY);
  
  DataRectangleDynamic layer_1_1 = new DataRectangleDynamic();  
  layer_1_1.describe(hor_size, vert_size);
  this_run.register_data(layer_1_1);
  
  DataRectangleDynamic layer_2_1 = new DataRectangleDynamic();  
  layer_2_1.describe(hor_size, vert_size);
  this_run.register_data(layer_2_1);
  
  DataRectangleDynamic layer_3_1 = new DataRectangleDynamic();  
  layer_3_1.describe(hor_size,vert_size);
  this_run.register_data(layer_3_1);
  
  DataRectangleDynamic layer_4_1 = new DataRectangleDynamic();  
  layer_4_1.describe(hor_size,vert_size);
  this_run.register_data(layer_4_1);
  
  DataRectangleDynamic layer_4_2 = new DataRectangleDynamic();  
  layer_4_2.describe(hor_size,vert_size);
  this_run.register_data(layer_4_2);
  
  this_run.register_output(margin_size, second_vert_size, layer_1_1);
  this_run.register_output(margin_size, margin_size, layer_3_1);
  this_run.register_output(second_hor_size, margin_size, layer_4_1);
  this_run.register_output(second_hor_size, second_vert_size, layer_4_2);
    
  CustomClickControl click_control_1 = new CustomClickControl(margin_size, second_vert_size, hor_size, vert_size);
  CustomClickControl click_control_2 = new CustomClickControl(margin_size, margin_size, hor_size, vert_size);
  NumericControl numeric_control_1 = new NumericControl (third_hor_size, margin_size, slider_width, vert_size);
  this_run.register_output(third_hor_size, margin_size, numeric_control_1.rect);
  NumericControl numeric_control_2 = new NumericControl (third_hor_size, second_vert_size, slider_width, vert_size);
  this_run.register_output(third_hor_size, second_vert_size, numeric_control_2.rect);
  
  this_run.register_control(click_control_1);
  this_run.register_control(click_control_2);
  this_run.register_control(numeric_control_1);
  this_run.register_control(numeric_control_2); 
 
  
  CustomWaveTransform transform_to_1 = new CustomWaveTransform(input_1, layer_1_1, click_control_1);
  this_run.register_transform(transform_to_1);
  transform_to_1.set_distance_coef(0.5f);
  
  NegationTransform transform_to_2 = new NegationTransform(layer_1_1, layer_2_1);
  this_run.register_transform(transform_to_2);
  
  CustomWaveTransform transform_to_3 = new CustomWaveTransform(layer_2_1, layer_3_1, click_control_2);
  this_run.register_transform(transform_to_3);
  
  SumOf2Transform transform_to_4_1 = new SumOf2Transform(input_1, layer_3_1, layer_4_1, numeric_control_1);
  this_run.register_transform(transform_to_4_1);
  
  SumOf2Transform transform_to_4_2 = new SumOf2Transform(layer_1_1, layer_3_1, layer_4_2, numeric_control_2);
  this_run.register_transform(transform_to_4_2);
  
  return input_1;
  
}
public void may_9_graph(MasterConfig this_run) {
    
  int side_size = 450;
  int margin_size = 5;
  int second_size = side_size+2*margin_size;
  int third_size = 2*side_size+3*margin_size;
  int slider_width = 50;

  size (third_size + slider_width + margin_size, third_size);
  
  DataRectangle input_1 = new DataRectangle();  
  input_1.describe(side_size, side_size, "2011-11-20 21h24_06.png");
  //input_1.img.filter(INVERT);
  //input_1.img.filter(GRAY);
  
  DataRectangleDynamic layer_1_1 = new DataRectangleDynamic();  
  layer_1_1.describe(side_size, side_size);
  this_run.register_data(layer_1_1);
  
  DataRectangleDynamic layer_2_1 = new DataRectangleDynamic();  
  layer_2_1.describe(side_size, side_size);
  this_run.register_data(layer_2_1);
  
  DataRectangleDynamic layer_3_1 = new DataRectangleDynamic();  
  layer_3_1.describe(side_size,side_size);
  this_run.register_data(layer_3_1);
  
  DataRectangleDynamic layer_4_1 = new DataRectangleDynamic();  
  layer_4_1.describe(side_size,side_size);
  this_run.register_data(layer_4_1);
  
  DataRectangleDynamic layer_4_2 = new DataRectangleDynamic();  
  layer_4_2.describe(side_size,side_size);
  this_run.register_data(layer_4_2);
  
  this_run.register_output(margin_size, second_size, layer_1_1);
  this_run.register_output(margin_size, margin_size, layer_3_1);
  //this_run.register_output(second_size, margin_size, layer_4_1);
  this_run.register_output(second_size, second_size, layer_4_2);
    
  CustomClickControl click_control_1 = new CustomClickControl(margin_size, second_size, side_size, side_size);
  //CustomClickControl click_control_2 = new CustomClickControl(margin_size, margin_size, side_size, side_size);
  NumericControl numeric_control_1 = new NumericControl (third_size, margin_size, slider_width, side_size);
  this_run.register_output(third_size, margin_size, numeric_control_1.rect);
  NumericControl numeric_control_2 = new NumericControl (third_size, second_size, slider_width, side_size);
  this_run.register_output(third_size, second_size, numeric_control_2.rect);
  
  this_run.register_control(click_control_1);
  //this_run.register_control(click_control_2);
  this_run.register_control(numeric_control_1);
  this_run.register_control(numeric_control_2); 
  numeric_control_2.value = 0.0f;
  numeric_control_2.render();
 
  
  CustomWaveTransform transform_to_1 = new CustomWaveTransform(layer_4_2, layer_1_1, click_control_1);
  this_run.register_transform(transform_to_1);
  transform_to_1.set_distance_coef(0.5f);
  
  /*NegationTransform transform_to_2 = new NegationTransform(layer_1_1, layer_2_1);
  this_run.register_transform(transform_to_2);*/
  
  /*CustomWaveTransform transform_to_3 = new CustomWaveTransform(layer_2_1, layer_3_1, click_control_2);
  this_run.register_transform(transform_to_3);*/
  
  SumOf2Transform transform_to_4_1 = new SumOf2Transform(input_1, layer_1_1, layer_3_1, numeric_control_1);
  this_run.register_transform(transform_to_4_1);
  
  SumOf2Transform transform_to_4_2 = new SumOf2Transform(input_1, layer_3_1, layer_4_2, numeric_control_2);
  this_run.register_transform(transform_to_4_2);
  
}
// Shapes for input and output must be equal for the time being

class NegationTransform extends Transform {
  
  DataRectangle input;
  DataRectangleDynamic output;
 
  NegationTransform(DataRectangle _input, DataRectangleDynamic _output) {
    input = _input;
    output = _output;
  } 
    
  public void apply() {
    assert output.rect_width == input.rect_width;
    assert output.rect_height == input.rect_height;
    output.img_write = input.img.get();  
    output.img_write.filter(INVERT);
  }
}
class NumericControl extends ClickControl {
  float value;
  
  DataRectangle rect;

  
  NumericControl(int _left_x, int _left_y, int _control_width, int _control_height) {
    super(_left_x, _left_y, _control_width, _control_height); 
    value = 0.5f;
    rect = new DataRectangle();
    rect.img = createImage(_control_width, _control_height, RGB);
    rect.rect_width = _control_width;
    rect.rect_height = _control_height;
    int c = color(180);
    for (int i = 0; i < _control_width; i++)
      for (int j = 0; j < control_height; j++)
        rect.img.set(i,j,c);
    
    render();    
  }
  
  public float transform_click_y_to_value (int click_y) {
    
    int inverted_click_y = control_height - click_y;
    float delta = inverted_click_y - 0.1f*control_height;
    if (delta < 0.0f) delta = 0.0f;
    delta = delta/(0.8f*control_height);
    if (delta > 1.0f) delta = 1.0f;
    return delta; 
  }
  
  public void click(int click_x, int click_y) {
    
    value = transform_click_y_to_value (click_y);
    println("control set to ", value);
 
    //println("frameRate is ", frameRate); 
    //frameRate(frameRate*5*value); 
    
    render();
  } 
  
  public void render() {
    int white = color(255);
    int black = color(0);
    int c = lerpColor(black, white, value);
    for (int i = control_width/5; i < 2*control_width/5; i++)
      for (int j = control_height/10; j < 9*control_height/10; j++)
        rect.img.set(i,j,c);
        
    for (int i = 3*control_width/5; i < 4*control_width/5; i++)
      for (int j = control_height/10; j < 9*control_height/10; j++) {
         float v = transform_click_y_to_value(j);
         if (v <= value)
           rect.img.set(i,j,black);
         else
           rect.img.set(i,j,white);
      }

  }
}
class OutputDataRectangle {

  int left_x;
  int left_y;
  DataRectangle data;
 
  OutputDataRectangle(int _left_x, int _left_y, DataRectangle _data) {
    left_x = _left_x;
    left_y = _left_y;
    data = _data;
  } 
  
}
// Shapes for input and output must be equal for the time being

class SumOf2Transform extends Transform {
  
  DataRectangle input1;
  DataRectangle input2;
  DataRectangleDynamic output;
  NumericControl control;
 
  SumOf2Transform(DataRectangle _input1, DataRectangle _input2, DataRectangleDynamic _output, NumericControl _control) {
    input1 = _input1;
    input2 = _input2;
    output = _output;
    control = _control;
  } 
    
  public void apply() {
    int output_width = output.rect_width;
    int output_height = output.rect_height;
    assert output_width == input1.rect_width;
    assert output_height == input1.rect_height;
    assert output_width == input2.rect_width;
    assert output_height == input2.rect_height;
    float interpolation = control.value;
    for (int i = 0; i < output_width; i++)
      for (int j = 0; j < output_height; j++) {
          int c1 = input1.get_color_point(i,j);
          int c2 = input2.get_color_point(i,j);
          int c = lerpColor(c1, c2, interpolation);
          output.set_color_point(i,j,c);
      }

  }
}
class Transform {
  
  public void apply () {
  }
}
  static public void main(String[] passedArgs) {
    String[] appletArgs = new String[] { "--full-screen", "--bgcolor=#666666", "--stop-color=#cccccc", "nonsquare_webcam_and_may_9_15" };
    if (passedArgs != null) {
      PApplet.main(concat(appletArgs, passedArgs));
    } else {
      PApplet.main(appletArgs);
    }
  }
}
